// apachemod/Screen.hh
// This file is part of Anyterm; see http://anyterm.org/
// (C) 2005-2007 Philip Endecott

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef Screen_hh
#define Screen_hh

#include <vector>

#include "Cell.hh"

#include "Array2d.hh"

#include "SharedObject.hh"


class Screen {
  int rows_;
  int cols_;
  int scrollback_;

  typedef Array2d< Cell, SharedObject::vector<Cell> > impl_t;
  impl_t impl;

public:
  Screen(int rows, int cols, int scrollback=0):
    rows_(rows), cols_(cols), scrollback_(scrollback),
    impl(cols,rows+scrollback),
    cursor_row(0), cursor_col(0), cursor_visible(true)
  {}

  int rows() const {
    return rows_;
  }

  int cols() const {
    return cols_;
  }

  int scrollback() const {
    return scrollback_;
  }

  Cell& operator()(int r, int c)
  {
    return impl(c,r+scrollback());
  }

  const Cell& operator()(int r, int c) const
  {
    return impl(c,r+scrollback());
  }

  void scroll_down(int top, int bottom, int n=1)
  {
    if (top==0 && bottom==rows()-1) {
      top = -scrollback();
    }
    for (int r=top; r<=bottom-n; ++r) {
      copy_line(r,r+n);
    }
    for (int r=bottom+1-n; r<=bottom; ++r) {
      clear_line(r);
    }
  }

  void scroll_up(int top, int bottom, int n=1)
  {
    for (int r=bottom; r>=top+n; --r) {
      copy_line(r,r-n);
    }
    for (int r=top; r<top+n; ++r) {
      clear_line(r);
    }
  }

  int cursor_row;
  int cursor_col;
  bool cursor_visible;


private:
  void copy_line(int dest, int src) {
    for (int c=0; c<cols(); ++c) {
      impl(c,dest+scrollback()) = impl(c,src+scrollback());
    }
  }

  void clear_line(int l) {
    for (int c=0; c<cols(); ++c) {
      impl(c,l+scrollback()) = Cell();  // FIXME this should probably use the terminal's current attributes 
                           // (e.g. background colour).
    }
  }
};



#endif
